@@ -1,7 +1,7 @@
-from django.contrib.auth import get_user_model, user_logged_in, user_logged_out
 from django.contrib.auth.tokens import default_token_generator
+from django.contrib.auth import get_user_model
+
 from rest_framework import generics, permissions, status, response, views
-from rest_framework.authtoken.models import Token
 from rest_framework.response import Response
 from rest_framework.reverse import reverse
 
@@ -44,7 +44,7 @@ class RootView(views.APIView):
         )
 
 
-class RegistrationView(utils.SendEmailViewMixin, generics.CreateAPIView):
+class RegistrationView(generics.CreateAPIView):
     """
     Use this endpoint to register new user.
     """
@@ -52,15 +52,24 @@ class RegistrationView(utils.SendEmailViewMixin, generics.CreateAPIView):
     permission_classes = (
         permissions.AllowAny,
     )
-    token_generator = default_token_generator
-    subject_template_name = 'activation_email_subject.txt'
-    plain_body_template_name = 'activation_email_body.txt'
 
     def perform_create(self, serializer):
-        instance = serializer.save()
-        signals.user_registered.send(sender=self.__class__, user=instance, request=self.request)
+        user = serializer.save()
+        signals.user_registered.send(sender=self.__class__, user=user, request=self.request)
         if settings.get('SEND_ACTIVATION_EMAIL'):
-            self.send_email(**self.get_send_email_kwargs(instance))
+            self.send_activation_email(user)
+        elif settings.get('SEND_CONFIRMATION_EMAIL'):
+            self.send_confirmation_email(user)
+
+    def send_activation_email(self, user):
+        email_factory = utils.UserActivationEmailFactory.from_request(self.request, user=user)
+        email = email_factory.create()
+        email.send()
+
+    def send_confirmation_email(self, user):
+        email_factory = utils.UserConfirmationEmailFactory.from_request(self.request, user=user)
+        email = email_factory.create()
+        email.send()
 
     def get_email_context(self, user):
         context = super(RegistrationView, self).get_email_context(user)
@@ -78,9 +87,7 @@ class LoginView(utils.ActionViewMixin, generics.GenericAPIView):
     )
 
     def action(self, serializer):
-        user = serializer.user
-        token, _ = Token.objects.get_or_create(user=user)
-        user_logged_in.send(sender=user.__class__, request=self.request, user=user)
+        token = utils.login_user(self.request, serializer.user)
         token_serializer_class = serializers.serializers_manager.get('token')
         return Response(
             data=token_serializer_class(token).data,
@@ -97,12 +104,11 @@ class LogoutView(views.APIView):
     )
 
     def post(self, request):
-        Token.objects.filter(user=request.user).delete()
-        user_logged_out.send(sender=request.user.__class__, request=request, user=request.user)
+        utils.logout_user(request)
         return response.Response(status=status.HTTP_204_NO_CONTENT)
 
 
-class PasswordResetView(utils.ActionViewMixin, utils.SendEmailViewMixin, generics.GenericAPIView):
+class PasswordResetView(utils.ActionViewMixin, generics.GenericAPIView):
     """
     Use this endpoint to send email to user with password reset link.
     """
@@ -110,26 +116,27 @@ class PasswordResetView(utils.ActionViewMixin, utils.SendEmailViewMixin, generic
     permission_classes = (
         permissions.AllowAny,
     )
-    token_generator = default_token_generator
-    subject_template_name = 'password_reset_email_subject.txt'
-    plain_body_template_name = 'password_reset_email_body.txt'
+
+    _users = None
 
     def action(self, serializer):
         for user in self.get_users(serializer.data['email']):
-            self.send_email(**self.get_send_email_kwargs(user))
+            self.send_password_reset_email(user)
         return response.Response(status=status.HTTP_204_NO_CONTENT)
 
     def get_users(self, email):
-        active_users = User._default_manager.filter(
-            email__iexact=email,
-            is_active=True,
-        )
-        return (u for u in active_users if u.has_usable_password())
+        if self._users is None:
+            active_users = User._default_manager.filter(
+                email__iexact=email,
+                is_active=True,
+            )
+            self._users = [u for u in active_users if u.has_usable_password()]
+        return self._users
 
-    def get_email_context(self, user):
-        context = super(PasswordResetView, self).get_email_context(user)
-        context['url'] = settings.get('PASSWORD_RESET_CONFIRM_URL').format(**context)
-        return context
+    def send_password_reset_email(self, user):
+        email_factory = utils.UserPasswordResetEmailFactory.from_request(self.request, user=user)
+        email = email_factory.create()
+        email.send()
 
 
 class SetPasswordView(utils.ActionViewMixin, generics.GenericAPIView):
@@ -148,6 +155,10 @@ class SetPasswordView(utils.ActionViewMixin, generics.GenericAPIView):
     def action(self, serializer):
         self.request.user.set_password(serializer.data['new_password'])
         self.request.user.save()
+
+        if settings.get('LOGOUT_ON_PASSWORD_CHANGE'):
+            utils.logout_user(self.request)
+
         return response.Response(status=status.HTTP_204_NO_CONTENT)
 
 
@@ -186,6 +197,11 @@ class ActivationView(utils.ActionViewMixin, generics.GenericAPIView):
         serializer.user.save()
         signals.user_activated.send(
             sender=self.__class__, user=serializer.user, request=self.request)
+        if settings.get('SEND_CONFIRMATION_EMAIL'):
+            email_factory = utils.UserConfirmationEmailFactory.from_request(
+                self.request, user=serializer.user)
+            email = email_factory.create()
+            email.send()
         return Response(status=status.HTTP_204_NO_CONTENT)
 
 
